{% extends "webpage/base.html" %}
{% load staticfiles %}
{% load crispy_forms_tags %}
{% block content %}
<div class="panel panel-default panel-fixed">
<div class="panel-heading"><h2>Agreement</h2></div>
<div class="panel-body">
    <div class="row">
        <div class="col-md-4">
            <div id="filter-container">
                {% crispy form %}
            </div>
      </div>

        <div class="col-md-8">
            <div id="tables-container">{% if tab1 %}<h3>{{ title1 }}</h3>{{ tab1 | safe }}{% endif %}
            {% if tab2 %}<hr/><h3>Precision, Recall and F1</h3>{{ tab2 | safe }}{% endif %}
            </div>
        </div>
    </div>
</div>
</div>
{% endblock %}